@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2015-2020 Kevin B. Hendricks, Stratford Ontario Canada 
+**  Copyright (C) 2015-2021 Kevin B. Hendricks, Stratford Ontario Canada 
 **  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
@@ -27,6 +27,7 @@
 #include <QHash>
 #include <QObject>
 #include <QUrl>
+#include <QPair>
 #include "ResourceObjects/OPFParser.h"
 #include "BookManipulation/XhtmlDoc.h"
 #include "ResourceObjects/Resource.h"
@@ -150,9 +151,9 @@ public:
      * Creates a new HTML Nav file with a basic nav structure.
      */
     HTMLResource *CreateEmptyNavFile(bool update_opf = false, 
-				     const QString &folderpath=QString("\\"), 
-				     const QString &navname=QString("nav.xhtml"),
-				     const QString &first_textdir=QString("\\"));
+                                     const QString &folderpath=QString("\\"), 
+                                     const QString &navname=QString("nav.xhtml"),
+                                     const QString &first_textdir=QString("\\"));
 
     /**
      * Creates a new HTMLResource file with a basic XHTML structure
@@ -244,7 +245,9 @@ public:
     static std::tuple<QString, QStringList> GetVideoInHTMLFileMapped(HTMLResource *html_resource);
     static std::tuple<QString, QStringList> GetAudioInHTMLFileMapped(HTMLResource *html_resource);
     static std::tuple<QString, std::pair<int,int> > GetWordCountsInHTMLFileMapped(HTMLResource *html_resource);
+    
 
+    bool CheckHTMLFilesForWellFormedness(const QList<HTMLResource*> html_resources);
     QList<HTMLResource *> GetNonWellFormedHTMLFiles();
     static std::pair<HTMLResource*, bool> ResourceWellFormedMap(HTMLResource * html_resource);
 
@@ -255,6 +258,8 @@ public:
      * If the merge fails, returns resource which caused the failure, otherwise returns null.
      */
     Resource *MergeResources(QList<Resource *> resources);
+    static QPair<QString, QString> UpdateAndExtractBodyInOneFile(Resource * resource, const QStringList &merged_bookpaths);
+
 
     QList <Resource *> GetAllResources();
 
