@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2015-2020 Kevin B. Hendricks, Stratford, Ontario Canada
+**  Copyright (C) 2015-2021 Kevin B. Hendricks, Stratford, Ontario Canada
 **  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
@@ -35,7 +35,7 @@
 #include "BookManipulation/HTMLMetadata.h"
 #include "BookManipulation/XhtmlDoc.h"
 #include "Importers/ImportHTML.h"
-#include "Misc/GumboInterface.h"
+#include "Parsers/GumboInterface.h"
 #include "Misc/HTMLEncodingResolver.h"
 #include "Misc/SettingsStore.h"
 #include "Misc/TempFolder.h"
@@ -106,7 +106,7 @@ QSharedPointer<Book> ImportHTML::GetBook(bool extract_metadata)
             HTMLResource * nav_resource = m_Book->CreateEmptyNavFile(true);
             m_Book->GetOPF()->SetNavResource(nav_resource);
             m_Book->GetOPF()->SetItemRefLinear(nav_resource, false);
-	    m_AddedBookPaths << nav_resource->GetRelativePath();
+            m_AddedBookPaths << nav_resource->GetRelativePath();
         }
     }
 
@@ -114,12 +114,11 @@ QSharedPointer<Book> ImportHTML::GetBook(bool extract_metadata)
     if (m_EpubVersion.startsWith('2')) {
         NCXResource* ncx_resource = m_Book->GetNCX();
         if (!ncx_resource) {
-            // add NCX using "toc.ncx" with id "ncx",  right beside OPF
-            // this is done to match the default text in a new OPF
+            // add NCX using "toc.ncx" with id "ncx" right beside the opf
             QString ncxbookpath = Utility::startingDir(m_Book->GetOPF()->GetRelativePath()) + "/toc.ncx";
             ncx_resource = m_Book->GetFolderKeeper()->AddNCXToFolder(m_EpubVersion, ncxbookpath);
-            // No Need to create a new manifest id for it as the empty epub2 already has an entry for it.
-            // and already has "ncx" entry on the spine
+            QString NCXId = m_Book->GetOPF()->AddNCXItem(ncx_resource->GetFullPath(),"ncx");
+            m_Book->GetOPF()->UpdateNCXOnSpine(NCXId);
             // And properly fill the empty ncx with the default contents to point to the first xhtml file
             QString first_xhtml_bookpath = m_AddedBookPaths.at(0);
             ncx_resource->FillWithDefaultTextToBookPath(m_EpubVersion, first_xhtml_bookpath);
@@ -220,25 +219,25 @@ void ImportHTML::UpdateFiles(HTMLResource *html_resource,
     foreach(QString target, TargetPaths) {
         if (target.indexOf(":") == -1) {
             std::pair<QString, QString> parts = Utility::parseRelativeHREF(target);
-	    QString target_file = hinfo.absolutePath() + "/" + parts.first;
-	    target_file = Utility::resolveRelativeSegmentsInFilePath(target_file, "/");
-	    if (!QFile::exists(target_file)) {
+            QString target_file = hinfo.absolutePath() + "/" + parts.first;
+            target_file = Utility::resolveRelativeSegmentsInFilePath(target_file, "/");
+            if (!QFile::exists(target_file)) {
                 html_updates[target_file] = "";
-	    } else {
-		QString extension = QFileInfo(target_file).suffix();
-	        // do not touch javascript links when importing html
-	        // even when they do exist as we do not import them
-	        if (extension == "js") {
+            } else {
+                QString extension = QFileInfo(target_file).suffix();
+                // do not touch javascript links when importing html
+                // even when they do exist as we do not import them
+                if (extension == "js") {
                     html_updates[target_file] = "";
-	        }
-		// we also do not touch links to *other* xhtml files
-		if ((target_file != currentpath) &&
-		    (extension == "htm" ||
-		     extension == "html" ||
-		     extension == "xhtml")) {
-                    html_updates[target_file] = "";
-		}
-	    }
+                }
+                // we also do not touch links to *other* xhtml files
+                if ((target_file != currentpath) &&
+                    (extension == "htm" ||
+                     extension == "html" ||
+                     extension == "xhtml")) {
+                     html_updates[target_file] = "";
+                }
+            }
         }
     }
     html_resource->SetText(PerformHTMLUpdates(newsource, newbookpath, html_updates, css_updates, currentpath, version)());
@@ -287,7 +286,7 @@ QHash<QString, QString> ImportHTML::LoadMediaFiles(const QStringList & file_path
             QString existing_book_path = m_Book->GetFolderKeeper()->GetBookPathByPathEnd(filename);
 
             if (m_IgnoreDuplicates && !existing_book_path.isEmpty()) {
-	        newpath = newpath = existing_book_path;
+                newpath = newpath = existing_book_path;
             } else {
                 Resource * resource = m_Book->GetFolderKeeper()->AddContentFileToFolder(fullfilepath);
                 newpath = resource->GetRelativePath();
@@ -319,10 +318,10 @@ QHash<QString, QString> ImportHTML::LoadStyleFiles(const QStringList & file_path
             QString filename = QFileInfo(file_path).fileName();
             QString fullfilepath  = QFileInfo(folder, file_path).absoluteFilePath();
             QString newpath;
-	    QString existing_book_path = m_Book->GetFolderKeeper()->GetBookPathByPathEnd(filename);
+            QString existing_book_path = m_Book->GetFolderKeeper()->GetBookPathByPathEnd(filename);
 
             if (m_IgnoreDuplicates && !existing_book_path.isEmpty()) {
-	        newpath = existing_book_path;
+                newpath = existing_book_path;
             } else {
                 Resource * resource = m_Book->GetFolderKeeper()->AddContentFileToFolder(fullfilepath);
                 newpath = resource->GetRelativePath();
