@@ -1,9 +1,9 @@
 /************************************************************************
 **
-**  Copyright (C) 2015-2019 Kevin B. Hendricks, Stratford, Ontario, Canada
+**  Copyright (C) 2015-2021 Kevin B. Hendricks, Stratford, Ontario, Canada
 **  Copyright (C) 2012-2015 John Schember <john@nachtimwald.com>
-**  Copyright (C) 2012 Dave Heiland
-**  Copyright (C) 2009-2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
+**  Copyright (C) 2012      Dave Heiland
+**  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
 **
@@ -39,7 +39,7 @@
 #include "Dialogs/ViewImage.h"
 #include "MainUI/FindReplace.h"
 #include "MainUI/TOCModel.h"
-#include "Misc/CSSInfo.h"
+#include "Parsers/CSSInfo.h"
 #include "Misc/PasteTarget.h"
 #include "Misc/SettingsStore.h"
 #include "Misc/ValidationResult.h"
@@ -103,10 +103,11 @@ public:
      * @param flags The flags used to modify window behavior.
      */
     MainWindow(const QString &openfilepath = QString(), 
-	       const QString version = QString(),
-	       bool is_internal = false, 
-	       QWidget *parent = 0, 
-	       Qt::WindowFlags flags = Qt::WindowFlags());
+               const QString version = QString(),
+               bool is_internal = false, 
+               QWidget *parent = 0, 
+               Qt::WindowFlags flags = Qt::WindowFlags());
+
     ~MainWindow();
 
     void maybe_fixup_dockwidget_geometry(QDockWidget * widget);
@@ -145,6 +146,10 @@ public:
      * @return List of valid selected HTML resources
      */
     QList <Resource *> GetValidSelectedHTMLResources();
+    QList <Resource *> GetValidSelectedCSSResources();
+
+    QList <Resource *> GetTabbedHTMLResources();
+    QList <Resource *> GetTabbedCSSResources();
 
     /**
      * Returns a list of all HTML resources in book browser order
@@ -152,7 +157,10 @@ public:
      * @return List of all HTML resources in book browser order
      */
     QList <Resource *> GetAllHTMLResources();
+    QList <Resource *> GetAllCSSResources();
 
+    QList <Resource *> GetOPFResource();
+    QList <Resource *> GetNCXResource();
 
     /**
      * Select resources in the Book Browser
@@ -1027,6 +1035,8 @@ private:
 
     QString m_mathjaxfolder;
 
+    bool m_FRVisible;
+
     /**
      * Holds all the widgets Qt Designer created for us.
      */
