@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2015-2020 Kevin B. Hendricks, Stratford Ontario Canada
+**  Copyright (C) 2015-2021 Kevin B. Hendricks, Stratford Ontario Canada
 **  Copyright (C) 2011      John Schember <john@nachtimwald.com>
 **
 **  This file is part of Sigil.
@@ -41,6 +41,8 @@
 #include "Misc/Utility.h"
 #include "sigil_constants.h"
 
+#define DBG if(0)
+
 #if !defined(Q_OS_WIN32) && !defined(Q_OS_MAC)
 # include <stdlib.h>
 #endif
@@ -58,6 +60,10 @@ SpellCheck *SpellCheck::instance()
 
 SpellCheck::SpellCheck()
 {
+    DBG qDebug() << "In SpellCheck Constructor";
+    m_primary.handle = NULL;
+    m_secondary.handle = NULL;
+
     // There is a considerable lag involved in loading the Spellcheck dictionaries
     QApplication::setOverrideCursor(Qt::WaitCursor);
     loadDictionaryNames();
@@ -78,11 +84,11 @@ SpellCheck::SpellCheck()
         }
     }
 
-    // Load the dictionary the user has selected if one was saved.
     QApplication::restoreOverrideCursor();
 
     UpdateLangCodeToDictMapping();
 
+    // Load the dictionary the user has selected
     // now open primary and secondary dictionaries
     SettingsStore settings;
     loadDictionary(settings.dictionary());
@@ -93,6 +99,7 @@ SpellCheck::SpellCheck()
 
 void SpellCheck::UpdateLangCodeToDictMapping()
 {
+    DBG qDebug() << "In UpdateLangCodeToDictMapping";
     m_langcode2dict.clear();
 
     // create language code to dictionary name mapping
@@ -105,12 +112,6 @@ void SpellCheck::UpdateLangCodeToDictMapping()
             m_langcode2dict[lc] = dname;
         }
     }
-#if 0
-    qDebug() << "langcode2dict";
-    foreach(QString lc, m_langcode2dict.keys()) {
-        qDebug() << lc << " -> " << m_langcode2dict[lc];
-    }
-#endif
 
     // make sure 2 letter mapping equivalent is properly set
     // for primary and secondary dictionaries
@@ -130,6 +131,7 @@ void SpellCheck::UpdateLangCodeToDictMapping()
 
 void SpellCheck::UnloadDictionary(const QString &dname)
 {
+    DBG qDebug() << "In UnloadDictionary";
     QMutexLocker locker(&mutex);
     if (m_opendicts.contains(dname)) {
         HDictionary hdic = m_opendicts[dname];
@@ -142,6 +144,7 @@ void SpellCheck::UnloadDictionary(const QString &dname)
 
 void SpellCheck::UnloadAllDictionaries()
 {
+    DBG qDebug() << "In UnloadAllDictionaries";
     foreach(QString name, m_opendicts.keys()) {
         UnloadDictionary(name);
     }
@@ -149,6 +152,7 @@ void SpellCheck::UnloadAllDictionaries()
 
 SpellCheck::~SpellCheck()
 {
+    DBG qDebug() << "In SpellCheck destructor";
     UnloadAllDictionaries();
 
     if (m_instance) {
@@ -159,6 +163,7 @@ SpellCheck::~SpellCheck()
 
 QStringList SpellCheck::userDictionaries()
 {
+    DBG qDebug() << "In userDictionaries";
     // Load the list of user dictionaries.
     QDir userDictDir(userDictionaryDirectory());
     QStringList user_dicts = userDictDir.entryList(QDir::Files | QDir::NoDotAndDotDot);
@@ -168,6 +173,7 @@ QStringList SpellCheck::userDictionaries()
 
 QStringList SpellCheck::dictionaries()
 {
+    DBG qDebug() << "In dictionaries";
     loadDictionaryNames();
     QStringList dicts;
     dicts = m_dictionaries.keys();
@@ -177,12 +183,14 @@ QStringList SpellCheck::dictionaries()
 
 QString SpellCheck::currentPrimaryDictionary() const
 {
+    DBG qDebug() << "In currentPrimaryDictionary";
     SettingsStore settings;
     return settings.dictionary();
 }
 
 bool SpellCheck::spell(const QString &word)
 {
+    DBG qDebug() << "In spell";
     QString dname = m_langcode2dict.value(HTMLSpellCheckML::langOf(word), "");
 
     // if no dictionary exists for this language treat it as correct
@@ -202,30 +210,23 @@ bool SpellCheck::spell(const QString &word)
 }
 
 
+// Speed here is very important as it is invoked by the XHTMLHighlighter2 code
+// and this is the limiting factor
 // spell check word without langcode info in Primary and Secondary Dictionaries
 bool SpellCheck::spellPS(const QString &word)
 {
-    SettingsStore settings;
-    QString dname = settings.dictionary();
-    // if primary dictionary does not exist simply return true (no error)
-    if (!m_opendicts.contains(dname)) {
-        return true;
-    }
-    HDictionary hdic = m_opendicts[dname];
-    bool res = hdic.handle->spell(hdic.codec->fromUnicode(Utility::getSpellingSafeText(word)).constData()) != 0;
-    res = res || isIgnored(word);
-    if (res) return res;
-    dname = settings.secondary_dictionary();
-    if (dname.isEmpty() || !m_opendicts.contains(dname)) return res;
-    hdic = m_opendicts[dname];
-    Q_ASSERT(hdic.codec != nullptr);
-    Q_ASSERT(hdic.handle != nullptr);
-    return hdic.handle->spell(hdic.codec->fromUnicode(Utility::getSpellingSafeText(word)).constData()) != 0;
+    if (!m_primary.handle) return true;
+    if(m_ignoredWords.contains(word)) return true;
+    bool res = m_primary.handle->spell(m_primary.codec->fromUnicode(Utility::getSpellingSafeText(word)).constData()) != 0;
+    if (res) return true;
+    if (!m_secondary.handle) return false;
+    return m_secondary.handle->spell(m_secondary.codec->fromUnicode(Utility::getSpellingSafeText(word)).constData()) != 0;
 }
 
 
 QStringList SpellCheck::suggest(const QString &word)
 {
+    DBG qDebug() << "In suggest";
     QStringList suggestions;
     char **suggestedWords;
     QString dname = m_langcode2dict.value(HTMLSpellCheckML::langOf(word), "");
@@ -248,57 +249,52 @@ QStringList SpellCheck::suggest(const QString &word)
 // suggesttions for word without langcode using Primary and Secondary Dictionaries
 QStringList SpellCheck::suggestPS(const QString &word)
 {
-    SettingsStore settings;
     QStringList suggestions;
     char **suggestedWords;
     char **suggestedWords2;
-    QString dname = settings.dictionary();
-    if (!m_opendicts.contains(dname)) return suggestions;
-    HDictionary hdic = m_opendicts[dname];
-    Q_ASSERT(hdic.codec != nullptr);
-    Q_ASSERT(hdic.handle != nullptr);
-    int count = hdic.handle->suggest(&suggestedWords, hdic.codec->fromUnicode(Utility::getSpellingSafeText(word)).constData());
+    if (!m_primary.handle) return suggestions;
+    int count = m_primary.handle->suggest(&suggestedWords, m_primary.codec->fromUnicode(Utility::getSpellingSafeText(word)).constData());
     int limit = count;
     if (limit > 4) limit = 4;
     for (int i = 0; i < limit; ++i) {
-        suggestions << hdic.codec->toUnicode(suggestedWords[i]);
+        suggestions << m_primary.codec->toUnicode(suggestedWords[i]);
     }
-    hdic.handle->free_list(&suggestedWords, count);
-    dname = settings.secondary_dictionary();
-    if (dname.isEmpty()) return suggestions;
-    hdic = m_opendicts[dname];
-    Q_ASSERT(hdic.codec != nullptr);
-    Q_ASSERT(hdic.handle != nullptr);
-    count = hdic.handle->suggest(&suggestedWords2, hdic.codec->fromUnicode(Utility::getSpellingSafeText(word)).constData());
+    m_primary.handle->free_list(&suggestedWords, count);
+    if (!m_secondary.handle) return suggestions; 
+    count = m_secondary.handle->suggest(&suggestedWords2, m_secondary.codec->fromUnicode(Utility::getSpellingSafeText(word)).constData());
     limit = count;
     if (limit > 4) limit = 4;
     for (int i = 0; i < limit; ++i) {
-        suggestions << hdic.codec->toUnicode(suggestedWords2[i]);
+        suggestions << m_secondary.codec->toUnicode(suggestedWords2[i]);
     }
-    hdic.handle->free_list(&suggestedWords2, count);
+    m_secondary.handle->free_list(&suggestedWords2, count);
     return suggestions;
 }
 
 
 void SpellCheck::clearIgnoredWords()
 {
+    DBG qDebug() << "In clearIgnoredWords";
     m_ignoredWords.clear();
 }
 
 
 void SpellCheck::ignoreWord(const QString &word)
 {
-    m_ignoredWords[word] = 1;
+    DBG qDebug() << "In ignoreWord";
+    m_ignoredWords.insert(word);
 }
 
 
 bool SpellCheck::isIgnored(const QString &word) {
-    return m_ignoredWords.value(word, 0);
+    DBG qDebug() << "In isIgnored";
+    return m_ignoredWords.contains(word);
 }
 
 
 void SpellCheck::addWordToDictionary(const QString &word, const QString &dname)
 {
+    DBG qDebug() << "In addWordToDictionary";
     if (dname.isEmpty()) return;
     if (m_opendicts.contains(dname)) {
         HDictionary hdic = m_opendicts[dname];
@@ -309,6 +305,7 @@ void SpellCheck::addWordToDictionary(const QString &word, const QString &dname)
 
 void SpellCheck::loadDictionary(const QString &dname)
 {
+    DBG qDebug() << "In loadDictionary: " << dname;
     QMutexLocker locker(&mutex);
     // If we don't have a dictionary we cannot continue.
     if (dname.isEmpty() || !m_dictionaries.contains(dname)) {
@@ -363,21 +360,29 @@ void SpellCheck::loadDictionary(const QString &dname)
         addWordToDictionary(word, dname);
     }
 
-    // finally add UserDictionary words to the Primary Dictionary only
+    // add UserDictionary words to the Primary Dictionary only
     if (dname == currentPrimaryDictionary()) {
         // Load in the words from the user dictionaries.
         foreach(QString word, allUserDictionaryWords()) {
             addWordToDictionary(word, dname);
         }
     }
+
+    SettingsStore settings;
+    // store the primary and secondary dictionary info for speed
+    if (dname == settings.dictionary()) {
+        m_primary = hdic;
+    }
+    else if (dname == settings.secondary_dictionary()) {
+        m_secondary = hdic;
+    }
     return;
 }
 
 
-
-
 void SpellCheck::setDictionary(const QString &dname, bool forceReplace)
 {
+    DBG qDebug() << "In setDictionary " << dname;
     // See if we are already using a hunspell object for this language.
     if (!forceReplace && m_opendicts.contains(dname)) {
         return;
@@ -390,6 +395,7 @@ void SpellCheck::setDictionary(const QString &dname, bool forceReplace)
 
 QString SpellCheck::getWordChars(const QString &lang)
 {
+    DBG qDebug() << "In getWordChars";
     QString dname;
     if (lang.isEmpty()) { 
         dname = currentPrimaryDictionary();
@@ -413,6 +419,7 @@ QString SpellCheck::getWordChars(const QString &lang)
 
 void SpellCheck::addToUserDictionary(const QString &word, QString dict_name)
 {
+    DBG qDebug() << "In AddToUserDictionary";
     // Adding to the user dictionary also marks the word as a correct spelling.
     if (word.isEmpty()) {
         return;
@@ -449,6 +456,7 @@ void SpellCheck::addToUserDictionary(const QString &word, QString dict_name)
 
 QStringList SpellCheck::allUserDictionaryWords()
 {
+    DBG qDebug() << "In allUserDictionaryWords";
     QStringList userWords;
     SettingsStore settings;
     foreach (QString dict_name, settings.enabledUserDictionaries()) {
@@ -459,6 +467,7 @@ QStringList SpellCheck::allUserDictionaryWords()
 
 QStringList SpellCheck::userDictionaryWords(QString dict_name)
 {
+    DBG qDebug() << "In userDictionaryWords";
     QStringList userWords;
     // Read each word from the user dictionary.
 
@@ -487,6 +496,7 @@ QStringList SpellCheck::userDictionaryWords(QString dict_name)
 
 void SpellCheck::dicDeltaWords(const QString &delta_path, QStringList & word_list)
 {
+    DBG qDebug() << "In dicDeltaWords";
     QFile deltaFile(delta_path);
     if (deltaFile.open(QIODevice::ReadOnly)) {
         QTextStream deltaStream(&deltaFile);
@@ -506,6 +516,7 @@ void SpellCheck::dicDeltaWords(const QString &delta_path, QStringList & word_lis
 
 void SpellCheck::loadDictionaryNames()
 {
+    DBG qDebug() << "In loadDictionaryNames";
     QStringList dictExts;
     dictExts << ".aff"
              << ".dic";
@@ -556,22 +567,25 @@ void SpellCheck::loadDictionaryNames()
 
 QString SpellCheck::dictionaryDirectory()
 {
+    DBG qDebug() << "In dictionaryDirectory";
     return Utility::DefinePrefsDir() + "/hunspell_dictionaries";
 }
 
 QString SpellCheck::userDictionaryDirectory()
 {
+    DBG qDebug() << "In userDictionaryDirectory";
     return Utility::DefinePrefsDir() + "/user_dictionaries";
 }
 
 QString SpellCheck::currentUserDictionaryFile()
 {
+    DBG qDebug() << "In currentUserDictionaryFile";
     SettingsStore settings;
     return userDictionaryDirectory() + "/" + settings.defaultUserDictionary();
 }
 
 QString SpellCheck::userDictionaryFile(QString dict_name)
 {
+    DBG qDebug() << "In userDictionaryFile";
     return userDictionaryDirectory() + "/" + dict_name;
 }
-
