@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2015-2020 Kevin B. Hendricks, Stratford Ontario Canada
+**  Copyright (C) 2015-2021 Kevin B. Hendricks, Stratford Ontario Canada
 **  Copyright (C) 2016-2020 Doug Massay
 **  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
@@ -59,6 +59,7 @@
 #include <QFileInfo>
 #include <QCollator>
 #include <QMenu>
+#include <QSet>
 #include <QVector>
 #include <QDebug>
 
@@ -729,7 +730,7 @@ void Utility::DisplayStdWarningDialog(const QString &warning_message, const QStr
     if (!detailed_text.isEmpty()) {
         message_box.setDetailedText(detailed_text);
     }
-    message_box.setStandardButtons(QMessageBox::Close);
+    message_box.setStandardButtons(QMessageBox::Ok);
     message_box.exec();
 }
 
@@ -788,20 +789,6 @@ bool Utility::has_non_ascii_chars(const QString &str)
 
 bool Utility::use_filename_warning(const QString &filename)
 {
-#if 0
-    const QString uri_delimiters = ":/?#[]@";
-    foreach(QChar c, uri_delimiters) {
-        if (filename.contains(c)) {
-            QMessageBox::warning(QApplication::activeWindow(),
-                   tr("Sigil"),
-                               tr("The requested file name contains one or more url/uri delimiter characters: "
-                  " : / ? # [ ] @  "
-                  "that should NOT appear in url/uri link targets such as filenames.\n\n"),
-                   QMessageBox::Ok);
-            return false;
-        }
-    }
-#endif
     if (has_non_ascii_chars(filename)) {
         return QMessageBox::Apply == QMessageBox::warning(QApplication::activeWindow(),
                 tr("Sigil"),
@@ -1024,48 +1011,6 @@ QStringList Utility::ZipInspect(const QString &zippath)
 
 // some utilities for working with absolute and book relative paths
 
-#if 0
-// brute force method
-QString Utility::longestCommonPath(const QStringList& filepaths, const QString& sep)
-{
-    // handle special cases
-    if (filepaths.isEmpty()) return QString();
-    if (filepaths.length() == 1) return QFileInfo(filepaths.at(0)).absolutePath() + sep;
-    
-    // split each path into its component segments
-    QList<QStringList> fpaths;
-    int minlen = -1;
-    foreach(QString apath, filepaths) {
-        QStringList segs = apath.split(sep);
-        int n = segs.length();
-        if (minlen == -1) minlen = n;
-        if (n < minlen) minlen = n;
-        fpaths.append(segs);
-    }
-
-    // now build up the results
-    QStringList res;
-    int numpaths = fpaths.length();
-    for(int i=0; i < minlen; i++) {
-        QString aseg = fpaths.at(0).at(i);
-        bool amatch = true;
-        int j = 1;
-        while(amatch && j < numpaths) {
-            amatch = (aseg == fpaths.at(j).at(i));
-            j++;
-        }
-        if (amatch) {
-            res << aseg;
-        } else {
-            break;
-        }
-    }
-    if (res.isEmpty()) return "";
-    return res.join(sep) + sep;
-}
-
-#else
-
 QString Utility::longestCommonPath(const QStringList& filepaths, const QString& sep)
 {
     if (filepaths.isEmpty()) return QString();
@@ -1084,8 +1029,6 @@ QString Utility::longestCommonPath(const QStringList& filepaths, const QString&
     return res.join(sep) + sep;
 }
 
-#endif
-
 
 // works with absolute paths and book (internal to epub) paths
 QString Utility::resolveRelativeSegmentsInFilePath(const QString& file_path, const QString &sep)
@@ -1216,11 +1159,11 @@ QString Utility::buildRelativeHREF(const QString &apath, const QString &afrag)
         if (id.startsWith("#")) {
             id = id.mid(1, -1);
         } else {
-	    qDebug() << "Warning: buildRelativeHREF has fragment that does not start with #" << afrag;
+            qDebug() << "Warning: buildRelativeHREF has fragment that does not start with #" << afrag;
         }
-	// technically fragments should be percent encoded if needed
-	id = URLEncodePath(id);
-	newhref = newhref + "#" + id;
+        // technically fragments should be percent encoded if needed
+        id = URLEncodePath(id);
+        newhref = newhref + "#" + id;
     }
     return newhref;
 }
@@ -1331,3 +1274,74 @@ QBrush Utility::ValidationResultBrush(const Val_Msg_Type &valres)
         }
     }
 }
+
+
+QString Utility::createCSVLine(const QStringList &data)
+{
+    QStringList csvline;
+    foreach(QString val, data) {
+        bool need_quotes = val.contains(',');
+        QString cval = "";
+        if (need_quotes) cval.append('"');
+        foreach(QChar c, val) {
+            if (c == '"') cval.append('"');
+            cval.append(c);
+        }
+        if (need_quotes) cval.append('"');
+        csvline.append(cval);
+    }
+    return csvline.join(',');
+}
+
+
+QStringList Utility::parseCSVLine(const QString &data)
+{
+    auto unquote_val = [](const QString &av) {
+        QString nv(av);
+        if (nv.startsWith('"')) nv = nv.mid(1);
+        if (nv.endsWith('"')) nv = nv.mid(0, nv.length()-1);
+        return nv;
+    };
+    
+    bool in_quote = false;
+    QStringList vals;
+    QString v;
+    int n = data.size();
+    int i = 0;
+    while(i < n) {
+        QChar c = data.at(i);
+        if (!in_quote) {
+            if (c == ',') {
+                vals.append(unquote_val(v.trimmed()));
+                v = "";
+            } else  {
+                v.append(c);
+                if (c == '"') in_quote = true;
+            }
+        } else {
+            v.append(c);
+            if (c == '"') {
+                if ((i+1 < n) && (data.at(i+1) == '"')) { 
+                    i++;
+                } else {
+                    in_quote = false;
+                }
+            }
+        }
+        i++;
+    }
+    if (!v.isEmpty()) vals.append(unquote_val(v.trimmed()));
+    return vals;
+}
+
+
+QString Utility::GenerateUniqueId(const QString &id, const QSet<QString>& Used)
+{
+    int cnt = 1;
+    QString new_id = id + "_" + QString::number(cnt);
+    while (Used.contains(new_id)) {
+        cnt++;
+        new_id = id + "_" + QString::number(cnt);
+    }
+    return new_id;
+}
