@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2016-2020 Kevin B. Hendricks, Stratford Ontario
+**  Copyright (C) 2016-2021 Kevin B. Hendricks, Stratford Ontario
 **
 **  This file is part of Sigil.
 **
@@ -29,7 +29,6 @@
 
 #include "Misc/Utility.h"
 #include "Misc/SettingsStore.h"
-#include "Misc/GumboInterface.h"
 #include "Misc/Landmarks.h"
 #include "BookManipulation/FolderKeeper.h"
 #include "ResourceObjects/Resource.h"
@@ -60,7 +59,7 @@ NavProcessor::NavProcessor(HTMLResource * nav_resource)
             "  </style>\n"
             "</head>\n"
             "<body epub:type=\"frontmatter\">\n"
-            "  <nav epub:type=\"toc\" id=\"toc\">\n"
+            "  <nav epub:type=\"toc\" id=\"toc\" role=\"doc-toc\">\n"
             "  </nav>\n"
             "  <nav epub:type=\"landmarks\" id=\"landmarks\" hidden=\"\">\n"
             "  </nav>\n"
@@ -81,10 +80,10 @@ NavProcessor::NavProcessor(HTMLResource * nav_resource)
         GumboAttribute* attr = gumbo_get_attribute(&node->v.element.attributes, "lang");
         if (attr) {
             lang = QString::fromUtf8(attr->value);
-	} else {
+        } else {
             attr = gumbo_get_attribute(&node->v.element.attributes, "xml:lang");
             if (attr) lang = QString::fromUtf8(attr->value);
-	}
+        }
     }
     m_language = lang;
 }
@@ -248,8 +247,8 @@ QList<NavTOCEntry> NavProcessor::GetNodeTOC(GumboInterface & gi, const GumboNode
                             te.lvl = lvl;
                             GumboAttribute* hrefattr = gumbo_get_attribute(&li_child->v.element.attributes, "href");
                             if (hrefattr) { 
-			        te.href = QString::fromUtf8(hrefattr->value);
-			    }
+                                te.href = QString::fromUtf8(hrefattr->value);
+                            }
                             te.title = Utility::DecodeXML(gi.get_local_text_of_node(li_child));
                             toclist.append(te);
                         } else if (li_child->v.element.tag == GUMBO_TAG_OL) {
@@ -271,7 +270,7 @@ QString NavProcessor::BuildTOC(const QList<NavTOCEntry> & toclist)
     bool initial = true;
     QString step = "  ";
     QString base = step.repeated(2);
-    res << "\n" + step + "<nav epub:type=\"toc\" id=\"toc\">\n";
+    res << "\n" + step + "<nav epub:type=\"toc\" id=\"toc\" role=\"doc-toc\">\n";
     res << base + "<h1>" + Landmarks::instance()->GetTitle("toc", m_language) + "</h1>\n";
     res << base + "<ol>\n";
     foreach(NavTOCEntry te, toclist) {
@@ -346,7 +345,7 @@ QString NavProcessor::BuildPageList(const QList<NavPageListEntry> & pagelist)
     QStringList res;
     QString step = "  ";
     QString base = step.repeated(3);
-    res << "\n" + step + "<nav epub:type=\"page-list\" id=\"page-list\" hidden=\"\">\n";
+    res << "\n" + step + "<nav epub:type=\"page-list\" id=\"page-list\" role=\"doc-pagelist\" hidden=\"\">\n";
     res << base + "<h1>" + Landmarks::instance()->GetTitle("page-list", m_language) + "</h1>\n";
     res << "\n" + base + "<ol>\n";
     foreach(NavPageListEntry pe, pagelist) {
@@ -514,9 +513,9 @@ void NavProcessor::AddLandmarkCode(const Resource *resource, QString new_code, b
             le.title = title;
             le.href = ConvertBookPathToNavRelative(resource->GetRelativePath());
             // special case the nav setting the toc semantic on itself
-	    if ((resource == m_NavResource) && (new_code == "toc")) {
+            if ((resource == m_NavResource) && (new_code == "toc")) {
                 le.href = "#toc";
-	    }
+            }
             landlist.append(le);
         }
     } else {
@@ -531,10 +530,11 @@ void NavProcessor::RemoveLandmarkForResource(const Resource * resource)
     QList<NavLandmarkEntry> landlist = GetLandmarks();
     QWriteLocker locker(&m_NavResource->GetLock());
     int pos = GetResourceLandmarkPos(resource, landlist);
-    if (pos > -1) {
+    while((pos > -1) && !landlist.isEmpty()) {
         landlist.removeAt(pos);
-        SetLandmarks(landlist);
+        pos = GetResourceLandmarkPos(resource, landlist);
     }
+    SetLandmarks(landlist);
 }
 
 int NavProcessor::GetResourceLandmarkPos(const Resource *resource, const QList<NavLandmarkEntry> & landlist)
@@ -769,7 +769,7 @@ QString NavProcessor::ConvertHREFToBookPath(const QString & nav_rel_href)
         // this link ends in the nav itself
         bookpath = Utility::URLEncodePath(m_NavResource->GetRelativePath());
         if (!fragment.isEmpty()) bookpath = bookpath + "#" + fragment; 
-	return bookpath;
+        return bookpath;
     }
     bookpath = Utility::buildBookPath(basepath, m_NavResource->GetFolder());
     bookpath = Utility::URLEncodePath(bookpath);
