@@ -139,6 +139,9 @@ public:
     void SetNavResource(HTMLResource* nav);
     HTMLResource* GetNavResource() const;
 
+    void UpdateGuideAfterMerge(QList<Resource*> &merged_resources, QHash<QString,QString> &section_id_map);
+    void UpdateGuideFragments(QHash<QString,QString> &idupdates);
+
  signals:
     void TextChanging();
     void LoadedFromDisk();
@@ -156,6 +159,7 @@ public slots:
     void AddResource(const Resource *resource);
 
     void RemoveResource(const Resource *resource);
+    void BulkRemoveResources(const QList<Resource *>resources);
 
     void AddGuideSemanticCode(HTMLResource *html_resource, QString code, bool toggle = true);
 
