@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2016-2019  Kevin B Hendricks, Stratford, Ontario, Canada
+**  Copyright (C) 2016-2021  Kevin B Hendricks, Stratford, Ontario, Canada
 **  Copyright (C) 2012       John Schember <john@nachtimwald.com>
 **  Copyright (C) 2012       Dave Heiland
 **  Copyright (C) 2012       Grant Drake
@@ -106,7 +106,7 @@ FlowTab::~FlowTab()
 
     if (!GetResourceWasDeleted()) {
         // was: disconnect(m_HTMLResource, SIGNAL(Modified()), this, SLOT(ResourceModified()));
-	disconnect(m_HTMLResource, 0, this , 0);
+        disconnect(m_HTMLResource, 0, this , 0);
     }
 
     m_WellFormedCheckComponent->deleteLater();
@@ -168,7 +168,7 @@ void FlowTab::DelayedInitialization()
     DelayedConnectSignalsToSlots();
 
     // sync Preview to where CodeView is now
-    // emit ScrollPreviewImmediately();
+    emit ScrollPreviewImmediately();
 
     // Cursor set in constructor
     QApplication::restoreOverrideCursor();
@@ -176,7 +176,7 @@ void FlowTab::DelayedInitialization()
 
 bool FlowTab::IsLoadingFinished()
 {
-    bool is_finished = true;
+    bool is_finished = false;
 
     if (m_wCodeView) {
         is_finished = m_wCodeView->IsLoadingFinished();
@@ -244,28 +244,26 @@ void FlowTab::ResourceModified()
         m_LastPosition = -1;
     }
 
-    DBG qDebug() << "FlowTab emitting UpdatePreview from ResourceModified";
-    EmitUpdatePreview();
+    if (IsLoadingFinished()) {
+        DBG qDebug() << "FlowTab in ResourceModified";
+        EmitUpdatePreview();
+    }
 }
 
 void FlowTab::HandleViewImage(const QUrl &url) 
 {
-    QString url_string = url.toString();
-
-    if (url_string.isEmpty()) {
-      return;
+    if (url.toString().isEmpty()) {
+        return;
     }
-    if (url_string.indexOf(':') != -1) return;
+    if (!url.isRelative()) return;
 
     // we have a relative url, so build an internal
-    // book: scheme url book:///bookpath#fragment
-    QString attpath = Utility::URLDecodePath(url_string);
-    QString dest_bookpath;
-    if (attpath.isEmpty()) return;
+    // book: scheme url book:///bookpath
+    if (url.path().isEmpty()) return;
 
     QString startdir = m_HTMLResource->GetFolder();
-    dest_bookpath = Utility::buildBookPath(attpath, startdir);
-    url_string = "book:///" + dest_bookpath;
+    QString dest_bookpath = Utility::buildBookPath(url.path(), startdir);
+    QString url_string = "book:///" + Utility::URLEncodePath(dest_bookpath);
     emit ViewImageRequest(QUrl(url_string));
 }
 
@@ -336,8 +334,10 @@ void FlowTab::EmitContentChanged()
 
 void FlowTab::EmitUpdatePreview()
 {
-    DBG qDebug() << "FlowTab emiting UpdatePreview from EmitUpdatePreview";
-    emit UpdatePreview();
+    if (IsLoadingFinished()) {
+        DBG qDebug() << "FlowTab emiting UpdatePreviewRequest from EmitUpdatePreview";
+        emit UpdatePreview();
+    }
 }
 
 void FlowTab::EmitUpdatePreviewImmediately()
@@ -413,6 +413,14 @@ bool FlowTab::RemoveFormattingEnabled()
     return false;
 }
 
+bool FlowTab::RemoveTagPairEnabled()
+{
+    if (m_wCodeView) {
+        return m_wCodeView->IsCutTagPairAllowed();
+    }
+    return false;
+}
+
 bool FlowTab::InsertClosingTagEnabled()
 {
     if (m_wCodeView) {
@@ -432,7 +440,7 @@ bool FlowTab::AddToIndexEnabled()
 bool FlowTab::MarkForIndexEnabled()
 {
     if (m_wCodeView) {
-        return true;
+        return m_wCodeView->textCursor().hasSelection();
     }
     return false;
 }
@@ -1001,6 +1009,13 @@ void FlowTab::RemoveFormatting()
     }
 }
 
+void FlowTab::RemoveTagPair()
+{
+    if (m_wCodeView) {
+        m_wCodeView->CutTagPair();
+    }
+}
+
 void FlowTab::ChangeCasing(const Utility::Casing casing)
 {
     if (m_wCodeView) {
