@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2015-2020 Kevin B. Hendricks, Stratford Ontario Canada
+**  Copyright (C) 2015-2021 Kevin B. Hendricks, Stratford Ontario Canada
 **  Copyright (C) 2009-2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
@@ -48,32 +48,19 @@ TabManager::TabManager(QWidget *parent)
     QTabWidget(parent),
     m_LastContentTab(NULL),
     m_TabsToDelete(QList<ContentTab*>()),
-    m_tabs_deletion_in_use(false)
+    m_tabs_deletion_in_use(false),
+    m_newTab(NULL)
 {
     QTabBar *tab_bar = new TabBar(this);
     setTabBar(tab_bar);
     connect(tab_bar, SIGNAL(TabBarClicked()),            this, SLOT(SetFocusInTab()));
     connect(tab_bar, SIGNAL(CloseOtherTabsRequest(int)), this, SLOT(CloseOtherTabs(int)));
-    connect(this, SIGNAL(currentChanged(int)),         this, SLOT(EmitTabChanged(int)));
-    connect(this, SIGNAL(tabCloseRequested(int)),      this, SLOT(CloseTab(int)));
+    connect(this, SIGNAL(currentChanged(int)),           this, SLOT(EmitTabChanged(int)));
+    connect(this, SIGNAL(tabCloseRequested(int)),        this, SLOT(CloseTab(int)));
     setDocumentMode(true);
 
     // re-enable tab drag and drop as a test to see if last commit helped
-#if 0    
-    // QTabBar has a bug when a user "presses and flicks" on a non current tab it will cause it 
-    // to setCurrentIndex() but during EmitTabChanged(int) it then may allows the resulting mouseMoveEvent
-    // be processed on the same index that is being set in setCurrentIndex which causes a crash.
-    // This bug makes it dangerous to enable dragging and dropping to move tabs in the QTabBar
-    // So default to non-movable but let a environment variable override this decision
-    if (qEnvironmentVariableIsSet("SIGIL_ALLOW_TAB_MOVEMENT")) {
-        setMovable(true);
-    } else {
-        setMovable(false);
-    }
-#else
     setMovable(true);
-#endif
-
     setTabsClosable(true);
     // setElideMode(Qt::ElideRight); this is the default after qt-5.6
     setUsesScrollButtons(true);
@@ -111,6 +98,19 @@ QList<Resource *> TabManager::GetTabResources()
     return tab_resources;
 }
 
+QList<Resource *> TabManager::GetTabResourcesOfType(Resource::ResourceType resource_type)
+{
+    QList <ContentTab *> tabs = GetContentTabs();
+    QList <Resource *> tab_resources;
+    foreach(ContentTab *tab, tabs) {
+        Resource* resource = tab->GetLoadedResource();
+        if (resource->Type() == resource_type) {
+            tab_resources.append(resource);
+        }
+    }
+    return tab_resources;
+}
+
 void TabManager::tabInserted(int index)
 {
     emit TabCountChanged();
@@ -177,11 +177,6 @@ void TabManager::ReopenTabs()
         CloseTabForResource(resource, true);
         OpenResource(resource, -1, -1, QString());
     }
-#if 0
-    foreach(Resource *resource, resources) {
-        OpenResource(resource, -1, -1, QString());
-    }
-#endif
     OpenResource(currentTab->GetLoadedResource(), -1, -1, QString());
 }
 
@@ -216,22 +211,22 @@ void TabManager::LinkClicked(const QUrl &url)
         if (url.hasFragment()) {
             fragment = url.fragment();
         }
-	QString dest_bookpath;
-	if (attpath.isEmpty()) {
-	    dest_bookpath = tab->GetLoadedResource()->GetRelativePath();
-	} else {
-	    QString startdir = tab->GetLoadedResource()->GetFolder();
-	    dest_bookpath = Utility::buildBookPath(attpath, startdir);
-	}
+        QString dest_bookpath;
+        if (attpath.isEmpty()) {
+            dest_bookpath = tab->GetLoadedResource()->GetRelativePath();
+        } else {
+            QString startdir = tab->GetLoadedResource()->GetFolder();
+            dest_bookpath = Utility::buildBookPath(attpath, startdir);
+        }
         url_string = "book:///" + Utility::buildRelativeHREF(dest_bookpath, fragment);
-	// QUrl will take care of encoding the url path
+    // QUrl will take care of encoding the url path
     } else {
         // we have a scheme and are absolute
         if (url.scheme() == "file") {
             if (url_string.contains("/#")) {
                 url_string.insert(url_string.indexOf("/#") + 1, tab->GetFilename());
             }
-	}
+        }
     }
 
     emit OpenUrlRequest(QUrl(url_string));
@@ -253,6 +248,7 @@ void TabManager::OpenResource(Resource *resource,
                           caret_location_to_scroll_to, fragment, grab_focus);
 
     if (new_tab) {
+        m_newTab = new_tab;
         AddNewContentTab(new_tab, precede_current_tab);
         emit ShowStatusMessageRequest("");
     } else {
@@ -365,10 +361,16 @@ void TabManager::EmitTabChanged(int new_index)
     ContentTab *current_tab = qobject_cast<ContentTab *>(currentWidget());
     // the result of the qobject_cast can be NULL and that is okay
     if (m_LastContentTab != current_tab) {
+        // qDebug() << "Emitting TabChanged Signal";
         ContentTab * prev_tab = m_LastContentTab;
         m_LastContentTab = current_tab;
         emit TabChanged(prev_tab, current_tab);
+        if (current_tab != m_newTab) {
+            // qDebug() << "Emitting UpdatePreviewAfterExistingTabSwitch";
+            emit UpdatePreviewAfterExistingTabSwitch();
+        }
     }
+    m_newTab = NULL;
 }
 
 
@@ -435,7 +437,7 @@ void TabManager::DeleteTab(ContentTab *tab_to_delete)
 
 
     while(!m_TabsToDelete.isEmpty()) {
-	ContentTab *tab = m_TabsToDelete.takeLast();
+    ContentTab *tab = m_TabsToDelete.takeLast();
 
         Q_ASSERT(tab);
 
@@ -451,13 +453,14 @@ void TabManager::DeleteTab(ContentTab *tab_to_delete)
         // Only the current tab is ever connected to the main ui
         // so do our own version of EmitTabChanged() only if needed
         // to disconnect and reconnect ui signals
-        ContentTab *new_tab = qobject_cast<ContentTab *>(currentWidget());
-        if (m_LastContentTab != new_tab) {
+        ContentTab *next_tab = qobject_cast<ContentTab *>(currentWidget());
+        if (m_LastContentTab != next_tab) {
             // move updating of m_LastContentTab to be upfront *before* emitting the signal
             ContentTab* prevtab = m_LastContentTab;
-            m_LastContentTab = new_tab;
+            m_LastContentTab = next_tab;
             // flow control is lost in following line
-            emit TabChanged(prevtab,  new_tab);
+            emit TabChanged(prevtab,  next_tab);
+            emit UpdatePreviewAfterExistingTabSwitch();
         }
         tab->deleteLater();
         m_tabs_deletion_in_use = !m_TabsToDelete.isEmpty();
@@ -556,7 +559,7 @@ bool TabManager::SwitchedToExistingTab(const Resource *resource,
 
 
             // manually update the Preview Location
-	    // flow_tab->EmitScrollPreviewImmediately();
+            // flow_tab->EmitScrollPreviewImmediately();
 
             return true;
         }
