@@ -1,7 +1,7 @@
 /************************************************************************
 **
-**  Copyright (C) 2019-2020 Kevin B. Hendricks, Stratford Ontario Canada
-**  Copyright (C) 2019-2020 Doug Massay
+**  Copyright (C) 2019-2021 Kevin B. Hendricks, Stratford Ontario Canada
+**  Copyright (C) 2019-2021 Doug Massay
 **
 **  This file is part of Sigil.
 **
@@ -36,11 +36,13 @@
 #include <QGuiApplication>
 #include <QDebug>
 
+#include "MainUI/MainApplication.h"
 #include "Misc/SettingsStore.h"
 #include "Misc/Utility.h"
 #include "sigil_constants.h"
 #include "ViewEditors/WebEngPage.h"
 #include "ViewEditors/ViewPreview.h"
+#include "ViewEditors/Overlay.h"
 
 #define DBG if(0)
 
@@ -56,38 +58,6 @@ const QString SET_PREVIEW_COLORS =
     "document.body.style.backgroundColor=\"%1\"; "
     "document.body.style.color=\"%2\";";
 
-#if 0
-    "var bodyStyle = window.getComputedStyle(document.body, null);"
-    "var bg = bodyStyle.backgroundColor;"
-    "var fg = bodyStyle.color;"
-    "console.log('bg', bg);"
-    "console.log('fg', fg);"
-    "var isRGB = bg.indexOf('rgb') == 0;"
-    "var isHEX = bg.indexOf('#') == 0;"
-    "console.log('isRGB', isRGB);"
-    "console.log('isHEX', isHEX);"
-     "if (isRGB) {"
-    "    var m = bg.match(/(\\d+){3}/g);"
-    "    if (m) var r = m[0], g = m[1], b = m[2];"
-    "    console.log('rgb', r, g, b);"
-    "}"
-    "if (isHEX) {"
-        "var c = bg.replace('#', '');"
-        "var r = parseInt(c.substr(0,2), 16);"
-        "var g = parseInt(c.substr(2,2), 16);"
-        "var b = parseInt(c.substr(4,2), 16);"
-    "    console.log('rgb', r, g, b);"
-    "}"
-    "if (typeof r != 'undefined') {"
-    "    if ((r*0.299 + g*0.587 + b*0.114) > 186) {"
-    "        document.body.style.color = '#000000';"
-    "        console.log('bg is light');" 
-    "    } else {"
-    "        document.body.style.color = '#FFFFFF';"
-    "        console.log('bg is dark');" 
-    "    }"
-    "}";
-#endif
 
 struct JSResult {
   QVariant res;
@@ -118,7 +88,8 @@ ViewPreview::ViewPreview(QWidget *parent)
       m_CaretLocationUpdate(QString()),
       m_CustomSetDocumentInProgress(false),
       m_pendingScrollToFragment(QString()),
-      m_LoadOkay(false)
+      m_LoadOkay(false),
+      m_overlay(new LoadingOverlay(this))
 {
     setPage(m_ViewWebPage);
     // Now handled in the WebEngPage constructor to be faster
@@ -180,10 +151,10 @@ void ViewPreview::CustomSetDocument(const QString &path, const QString &html)
         // This next line really causes problems as it happens to interfere with later loading
         // StoreCurrentCaretLocation();
  
-	// keep memory footprint small clear any caches when a new page loads
-	if (url().toLocalFile() != path) {
-	    page()->profile()->clearHttpCache();
-	} 
+        // keep memory footprint small clear any caches when a new page loads
+        if (url().toLocalFile() != path) {
+            page()->profile()->clearHttpCache();
+        } 
     }
 
     m_isLoadFinished = false;
@@ -194,7 +165,15 @@ void ViewPreview::CustomSetDocument(const QString &path, const QString &html)
     // Sigil as well as catering for section splits etc.
     QString replaced_html = html;
     replaced_html = replaced_html.replace("<html>", "<html xmlns=\"http://www.w3.org/1999/xhtml\">");
-    setContent(replaced_html.toUtf8(), "application/xhtml+xml;charset=UTF-8", QUrl::fromLocalFile(path));
+    MainApplication *mainApplication = qobject_cast<MainApplication *>(qApp);
+    QString key = Utility::CreateUUID();
+    mainApplication->saveInPreviewCache(key, replaced_html);
+    QUrl tgturl = QUrl::fromLocalFile(path);
+    tgturl.setScheme("sigil");
+    tgturl.setHost("");
+    tgturl.setQuery("sigilpreview=" + key); 
+    page()->load(tgturl);
+    // setContent(replaced_html.toUtf8(), "application/xhtml+xml;charset=UTF-8", QUrl::fromLocalFile(path));
 }
 
 bool ViewPreview::IsLoadingFinished()
@@ -305,11 +284,12 @@ void ViewPreview::LoadingStarted()
 // and set it to finished when progress hits 100
 void ViewPreview::LoadingProgress(int progress)
 {
-  DBG qDebug() << "Loading progress " << progress;
-  if (progress >= 100 && !m_CustomSetDocumentInProgress) {
-      m_isLoadFinished = true;
-      m_LoadOkay = true;
-  }
+    DBG qDebug() << "Loading progress " << progress;
+    if (progress >= 100 && !m_CustomSetDocumentInProgress) {
+        m_isLoadFinished = true;
+        m_LoadOkay = true;
+    }
+    emit ViewProgress(progress);
 }
 
 void ViewPreview::UpdateFinishedState(bool okay)
@@ -322,7 +302,16 @@ void ViewPreview::UpdateFinishedState(bool okay)
     DBG qDebug() << "UpdateFinishedState with okay " << okay;
     // m_isLoadFinished = true;
     m_LoadOkay = okay;
-    emit DocumentLoaded();
+}
+
+void ViewPreview::HideOverlay()
+{
+    m_overlay->hide();
+}
+
+void ViewPreview::ShowOverlay()
+{
+    m_overlay->show();
 }
 
 QVariant ViewPreview::EvaluateJavascript(const QString &javascript)
@@ -347,7 +336,7 @@ QVariant ViewPreview::EvaluateJavascript(const QString &javascript)
         delete pres;
     } else {
         qDebug() << "Error: VP EvaluateJavascript timed out";
-	qDebug() << "   ... intentionally leaking the JSResult structure";
+        qDebug() << "   ... intentionally leaking the JSResult structure";
     }
     return res;
 }
@@ -394,8 +383,9 @@ void ViewPreview::WebPageJavascriptOnLoad()
         } else {
             executeCaretUpdateInternal();
         }
-	m_CustomSetDocumentInProgress = false;
+        m_CustomSetDocumentInProgress = false;
     }
+    emit DocumentLoaded();
 }
 
 QString ViewPreview::GetElementSelectingJS_NoTextNodes(const QList<ElementIndex> &hierarchy) const
@@ -497,15 +487,6 @@ QString ViewPreview::GetElementSelectingJS_WithTextNode(const QList<ElementIndex
 
 bool ViewPreview::ExecuteCaretUpdate()
 {
-    // The following is no longer needed as we disable javascripts from running until load finished
-    // happens, otherwise this will be ignored.
-#if 0
-    // Currently certain actions in Sigil result in a document being loaded multiple times
-    // in response to the signals. Only proceed with moving the caret if all loads are finished.
-    if (m_pendingLoadCount > 0) {
-        return false;
-    }
-#endif
 
     // If there is no caret location update pending...
     if (m_CaretLocationUpdate.isEmpty()) {
@@ -541,69 +522,3 @@ void ViewPreview::ConnectSignalsToSlots()
     connect(page(), SIGNAL(loadProgress(int)), this, SLOT(LoadingProgress(int)));
     connect(page(), SIGNAL(linkHovered(const QString &)), this, SLOT(LinkHovered(const QString &)));
 }
-
-
-// some pieces to keep for the future just in case
-#if 0
-
-void ViewPreview::InspectElement()
-{
-    page()->triggerAction(QWebEnginePage::InspectElement);
-    connect(m_InspectElement,    SIGNAL(triggered()),  this, SLOT(InspectElement()));
-}
-
-struct HTMLResult {
-  QString res;
-  bool finished;
-  HTMLResult() : res(QString("")), finished(false) {}
-  HTMLResult(HTMLResult * pRes) : res(pRes->res),  finished(pRes->finished) {}
-  bool isFinished() { return finished; }
-};
-
-struct SetToHTMLResultFunctor {
-    HTMLResult * pres;
-    SetToHTMLResultFunctor(HTMLResult * pres) : pres(pres) {}
-    void operator()(const QString &result) {
-        pres->res = result;
-        pres->finished = true;
-    }
-};
-
-QString ViewPreview::GetHTML() const 
-{
-    HTMLResult * pres = new HTMLResult();
-    page()->toHtml(SetToHTMLResultFunctor(pres));
-    while(!pres->isFinished()) {
-        qApp->processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::ExcludeSocketNotifiers, 100);
-    }
-    QString res = pres->res;
-    delete pres;
-    return res;
-}
-
-    //for inside of the ViewPreview destructor
-    if (m_InspectElement) {
-        delete m_InspectElement;
-        m_InspectElement = 0;
-    }
-
-QString ViewPreview::GetSelectedText()
-{
-    QString javascript = "window.getSelection().toString();";
-    return EvaluateJavascript(javascript).toString();
-}
-
-void ViewPreview::HighlightPosition()
-{
-    page()->runJavaScript("document/documentElement.contentEditable = true", QWebEngineScript::ApplicationWorld);
-    page()->triggerAction(QWebEnginePage::SelectEndOfBlock);
-    page()->runJavaScript("document/documentElement.contentEditable = false", QWebEngineScript::ApplicationWorld);
-}
-
-void ViewPreview::copy()
-{
-    page()->triggerAction(QWebEnginePage::Copy);
-}
-
-#endif
-
