@@ -1,7 +1,7 @@
 /************************************************************************
 **
-**  Copyright (C) 2018-2020  Kevin B. Hendricks, Stratford Ontario Canada
-**  Copyright (C) 2019-2020  Doug Massay
+**  Copyright (C) 2018-2021  Kevin B. Hendricks, Stratford Ontario Canada
+**  Copyright (C) 2019-2021  Doug Massay
 **  Copyright (C) 2009-2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
@@ -21,7 +21,7 @@
 **
 *************************************************************************/
 
-#include "Misc/EmbeddedPython.h"
+#include "EmbedPython/EmbeddedPython.h"
 #include <iostream>
 
 #include <QtCore/QCoreApplication>
@@ -42,6 +42,10 @@
 #include <QFontMetrics>
 #include <QtWebEngineWidgets/QWebEngineProfile>
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
+#include <QWebEngineUrlScheme>
+#endif
+
 #include "Misc/PluginDB.h"
 #include "Misc/UILanguage.h"
 #include "MainUI/MainApplication.h"
@@ -53,10 +57,10 @@
 #include "Misc/UpdateChecker.h"
 #include "Misc/Utility.h"
 #include "Misc/URLInterceptor.h"
+#include "Misc/URLSchemeHandler.h"
 #include "sigil_constants.h"
 #include "sigil_exception.h"
 
-
 #ifdef Q_OS_WIN32
 #include <QtWidgets/QPlainTextEdit>
 static const QString WIN_CLIPBOARD_ERROR = "QClipboard::setMimeData: Failed to set data on clipboard";
@@ -245,7 +249,7 @@ void MessageHandler(QtMsgType type, const QMessageLogContext &context, const QSt
         ts << qt_debug_message << Qt::endl;
 #else
         ts << qt_debug_message << endl;
-#endif	
+#endif  
     }
 }
 
@@ -281,7 +285,7 @@ void setupHighDPI()
         QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling, false);
         foreach(QString v, env_vars) {
             bool irrel = qunsetenv(v.toUtf8().constData());
-	    Q_UNUSED(irrel);
+        Q_UNUSED(irrel);
         }
     }
 }
@@ -303,11 +307,12 @@ int main(int argc, char *argv[])
         QStringList env_vars = {"QT_QPA_PLATFORMTHEME", "QT_STYLE_OVERRIDE"};
         foreach(QString v, env_vars) {
             bool irrel = qunsetenv(v.toUtf8().constData());
-	    Q_UNUSED(irrel);
+        Q_UNUSED(irrel);
         }
-}
+    }
 #endif
 
+
 #ifndef QT_DEBUG
     qInstallMessageHandler(MessageHandler);
 #endif
@@ -318,6 +323,18 @@ int main(int argc, char *argv[])
     QCoreApplication::setApplicationName("sigil");
     QCoreApplication::setApplicationVersion(SIGIL_VERSION);
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
+    // register the our own url scheme (this is required since Qt 5.12)
+    QWebEngineUrlScheme sigilScheme("sigil");
+    sigilScheme.setFlags(QWebEngineUrlScheme::SecureScheme |
+                         QWebEngineUrlScheme::LocalScheme |
+                         QWebEngineUrlScheme::LocalAccessAllowed |
+                         QWebEngineUrlScheme::ContentSecurityPolicyIgnored);
+    // sigilScheme.setSyntax(QWebEngineUrlScheme::Syntax::Host);
+    sigilScheme.setSyntax(QWebEngineUrlScheme::Syntax::Path);
+    QWebEngineUrlScheme::registerScheme(sigilScheme);
+#endif
+
 #ifndef Q_OS_MAC
     setupHighDPI();
 #endif
@@ -335,12 +352,10 @@ int main(int argc, char *argv[])
 #endif
 
     // On recent processors with multiple cores this leads to over 40 threads at times
-#if 0   
     // We prevent Qt from constantly creating and deleting threads.
     // Using a negative number forces the threads to stay around;
     // that way, we always have a steady number of threads ready to do work.
-    QThreadPool::globalInstance()->setExpiryTimeout(-1);
-#endif
+    // QThreadPool::globalInstance()->setExpiryTimeout(-1);
 
     // QtWebEngine may need this
     QCoreApplication::setAttribute(Qt::AA_ShareOpenGLContexts);
@@ -446,8 +461,7 @@ int main(int argc, char *argv[])
         settings.setOriginalUIFont(f.toString());
         if (!settings.uiFont().isEmpty()) {
             QFont font;
-            if (font.fromString(settings.uiFont()))
-                QApplication::setFont(font);
+            if (font.fromString(settings.uiFont())) QApplication::setFont(font);
         }
 #ifndef Q_OS_MAC
         // redo on a timer to ensure in all cases
@@ -514,6 +528,10 @@ int main(int argc, char *argv[])
 #endif
 #endif
 
+        // Install our own URLSchemeHandler for QtWebEngine to bypass 2mb url limit
+        URLSchemeHandler handlescheme; 
+        QWebEngineProfile::defaultProfile()->installUrlSchemeHandler("sigil", &handlescheme);
+
         // Install our own URLInterceptor for QtWebEngine to protect
         // against bad file:: urls
         URLInterceptor* urlint = new URLInterceptor();
@@ -525,8 +543,11 @@ int main(int argc, char *argv[])
 
         // Needs to be created on the heap so that
         // the reply has time to return.
-        UpdateChecker *checker = new UpdateChecker(&app);
-        checker->CheckForUpdate();
+        // Skip if compile-time define or runtime env var is set.
+        if ((!DONT_CHECK_FOR_UPDATES) && (!qEnvironmentVariableIsSet("SKIP_SIGIL_UPDATE_CHECK"))) {
+            UpdateChecker *checker = new UpdateChecker(&app);
+            checker->CheckForUpdate();
+        }
 
         // select the icon theme to use
         QString RCCResourcePath;
@@ -544,21 +565,21 @@ int main(int argc, char *argv[])
             RCCResourcePath = sigil_share_root + "/iconthemes";
         }
 #endif
-	QString icon_theme = settings.uiIconTheme();
+        QString icon_theme = settings.uiIconTheme();
         // First check if user wants the Custom Icon Theme
-	if (icon_theme == "custom") {
-	    // it must exist and be loadable
-	    QString CustomRCCPath = Utility::DefinePrefsDir() + "/" + CUSTOM_ICON_THEME_FILENAME;
+        if (icon_theme == "custom") {
+            // it must exist and be loadable
+            QString CustomRCCPath = Utility::DefinePrefsDir() + "/" + CUSTOM_ICON_THEME_FILENAME;
             bool loaded = false;
-	    if (QFileInfo(CustomRCCPath).exists()) {
-		loaded = QResource::registerResource(Utility::DefinePrefsDir() + "/" + CUSTOM_ICON_THEME_FILENAME);
-	    }
-	    if (!loaded) {
-		// revert to using main
-		icon_theme = "main";
-		settings.setUIIconTheme("main");
-	    }
-	}
+            if (QFileInfo(CustomRCCPath).exists()) {
+                loaded = QResource::registerResource(Utility::DefinePrefsDir() + "/" + CUSTOM_ICON_THEME_FILENAME);
+            }
+            if (!loaded) {
+                // revert to using main
+                icon_theme = "main";
+                settings.setUIIconTheme("main");
+            }
+        }
         // qDebug() << RCCResourcePath;
         QResource::registerResource(RCCResourcePath + "/" + icon_theme + ".rcc");
 
@@ -598,7 +619,6 @@ int main(int argc, char *argv[])
 
             // Create the Application Menu
             QMenu *app_menu = new QMenu("Sigil");
-
             QIcon icon;
 
             // Quit
